# This is a test script to analyze air table data
# matplotlib is used to plot and fit 

# Load python modules.
import pylab
import numpy
from scipy.optimize import curve_fit

# set interactive session or the canvas will freeze the execution
pylab.ion()

# this script dir, just to use absolute paths
import os
this_dir = os.path.dirname(os.path.abspath(__file__))

# Load the data (produced by plasduino/videoclick)
data_filepath = os.path.join(this_dir, 'data/data_air_table.txt')
n, t, x1, y1, x2, y2 = pylab.loadtxt(data_filepath, unpack = True)

# Define disk masses and uncertainty
M2 = 32.83 # [g]
M1 = 48.28
dM =  0.01

# Select the id that separate data before impact
# from data after: Point Of Closest Approach (POCA) 
poca_id = 6

# Plot table view from above and save
pylab.figure(1)
pylab.plot(x1, y1, 'o', x2,y2, 'o')

pylab.xlabel('x [cm]')
pylab.ylabel('y [cm]')
pylab.savefig('plot_air_table_topview.png')
pylab.show()
pylab.close()

# Setup the linear fit function
def f(x, m, q):
    return m*x + q

# Function to fit data and get speed
def getSpeed(x, y, show = False, verbose = False):
    popt, pcov = curve_fit(f, x, y, numpy.array([1.,1.]))
    m, q   = popt
    dm, dq = numpy.sqrt(pcov.diagonal())
    dy = numpy.array(len(y)*[0.15]) # taken from rms of non-moving disk
    chisq = (((y - f(x, m, q))/dy)**2).sum()
    ndof = len(x) - 2
    if verbose:
        print('m = %f +- %f' % (m, dm))
        print('q = %f +- %f' % (q, dq))
        print('chisq/ndof = %f/%d' % (chisq, ndof))
    if show:        
        pylab.errorbar(xx, yy, xerr=0, yerr=0.15, fmt='o')
        pylab.plot(xx, f(xx, m, q), 'r')
        pylab.show()
        raw_input("press Enter to continue")
        pylab.close()
    return m, dm, q, dq

#
# Do all fits
#

# X vs t
print('Fitting disk 2 - X before ')
xx = t[:poca_id]
yy = x2[:poca_id]
# this disk is not moving - let's set v = 0 and eval position rms
vx2  = 0
dvx2 = 0
n  = len(yy)
qx2  = yy.sum()/n 
dqx2 = numpy.sqrt(((yy - qx2)**2/(n-1)).sum())
print('Setting v=0 and x = %.2f +- %.2f' % (qx2, dqx2))

print('Fitting disk 2 - X after ')
xx = t[(poca_id+1):]
yy = x2[(poca_id+1):]
vx2p, dvx2p, qx2p, dq = getSpeed(xx,yy)

print('Fitting disk 1 - X before ')
xx = t[:poca_id]
yy = x1[:poca_id]
vx1, dvx1, qx1, dq = getSpeed(xx,yy)

print('Fitting disk 1 - X after ')
xx = t[(poca_id+1):]
yy = x1[(poca_id+1):]
vx1p, dvx1p, qx1p, dq = getSpeed(xx,yy)

# Y vs t
print('Fitting disk 2 - Y before ')
xx = t[:poca_id]
yy = y2[:poca_id]
# this disk is not moving - let's set v = 0 and eval position rms
vy2  = 0
dvy2 = 0
n  = len(yy)
qy2  = yy.sum()/n 
dqy2 = numpy.sqrt(((yy - qy2)**2/(n-1)).sum())
print('Setting v=0 and y = %.2f +- %.2f' % (qy2, dqy2))

print('Fitting disk 2 - Y after ')
xx = t[(poca_id+1):]
yy = y2[(poca_id+1):]
vy2p, dvy2p, qy2p, dq = getSpeed(xx,yy)

print('Fitting disk 1 - Y before ')
xx = t[:poca_id]
yy = y1[:poca_id]
vy1, dvy1, qy1, dq = getSpeed(xx,yy)

print('Fitting disk 1 - Y after ')
xx = t[(poca_id+1):]
yy = y1[(poca_id+1):]
vy1p, dvy1p, qy1p, dq = getSpeed(xx,yy)

#
# Put together all speeds and evaluate momentum
#

print('\n\tMomentum X')
px  = vx1*M1 + vx2*M2 
pxp = vx1p*M1 + vx2p*M2 
dpx  = numpy.sqrt( (dvx1*M1)**2  + (dvx2*M2)**2  + 
                   (vx1*dM)**2   + (vx2*dM)**2  )
dpxp = numpy.sqrt( (dvx1p*M1)**2 + (dvx2p*M2)**2 + 
                   (vx1p*dM)**2  + (vx2p*dM)**2 )
print('px_before              \t= %.0f +- %.0f' % (px, dpx))
print('px_after               \t= %.0f +- %.0f' % (pxp, dpxp))
print('px_after - px_before   \t= %.0f +- %.0f' % \
          (pxp-px,  numpy.sqrt(dpx**2+dpxp**2)))
print('|px_after/px_before-1| \t= %.1f%%' % (100*abs(pxp/px-1)))

print('\n\tMomentum Y')
py  = vy1*M1 + vy2*M2 
pyp = vy1p*M1 + vy2p*M2 
dpy  = numpy.sqrt( (dvy1*M1)**2  + (dvy2*M2)**2  + 
                   (vy1*dM)**2   + (vy2*dM)**2  )
dpyp = numpy.sqrt( (dvy1p*M1)**2 + (dvy2p*M2)**2 + 
                   (vy1p*dM)**2  + (vy2p*dM)**2 )
print('py_before              \t= %.0f +- %.0f' % (py, dpy))
print('py_after               \t= %.0f +- %.0f' % (pyp, dpyp))
print('py_after - py_before   \t= %.0f +- %.0f' % 
      (pyp-py,  numpy.sqrt(dpy**2+dpyp**2)))
print('|py_after/py_before-1| \t= %.1f%%' % (100*abs(pyp/py-1)))

#
# Plot X and Y views vs Time with fit
#

pylab.figure(2) 

pylab.subplot(211)
pylab.plot(t,x2, 'b^',\
               t[:poca_id], f(t[:poca_id], vx2, qx2), 'g-', \
               t[:poca_id], x2[:poca_id], 'g^',\
               t[(poca_id+1):], f(t[(poca_id+1):], vx2p, qx2p), 'r-', \
               t[(poca_id+1):], x2[(poca_id+1):], 'r^',\
               t,x1, 'co', \
               t[:poca_id], f(t[:poca_id], vx1, qx1), 'm-', \
               t[:poca_id], x1[:poca_id], 'mo', \
               t[(poca_id+1):], f(t[(poca_id+1):], vx1p, qx1p), 'y-',  \
               t[(poca_id+1):], x1[(poca_id+1):], 'yo')
pylab.title('Disk position vs time with fit')
pylab.xlabel('')
pylab.ylabel('x [cm]')

pylab.subplot(212)
pylab.plot(t,y2, 'b^', \
               t[:poca_id], f(t[:poca_id], vy2, qy2), 'g-', \
               t[:poca_id], y2[:poca_id], 'g^',\
               t[(poca_id+1):], f(t[(poca_id+1):], vy2p, qy2p), 'r-', \
               t[(poca_id+1):], y2[(poca_id+1):], 'r^',\
               t,y1, 'co', \
               t[:poca_id], f(t[:poca_id], vy1, qy1), 'm-', \
               t[:poca_id], y1[:poca_id], 'mo', \
               t[(poca_id+1):], f(t[(poca_id+1):], vy1p, qy1p), 'y-', \
               t[(poca_id+1):], y1[(poca_id+1):], 'yo')
pylab.xlabel('t [s]')
pylab.ylabel('y [cm]')

pylab.savefig('plot_air_table_xytime.png')
pylab.show()


