# This is a script to analyze temperature profile of a metal bar 
# kept at constant (different) temperatures at its extreme
# matplotlib is used to plot and scipy for fitting
# 

# Load python modules.
import pylab
import numpy
from scipy.optimize import curve_fit
import scipy.odr.odrpack as odrpack

# set interactive session or the canvas will freeze the execution
pylab.ion()

# input data: temperature y [C] and positions x [cm]
x = numpy.array([ 2.5,   5.0,   7.5,  10.,   12.5,  15.,   17.5,  20.,   
                  22.5,  25.,   27.5,  30.,   32.5,  35.,   37.50], 'd')
y = numpy.array([29.04, 28.61, 28.18, 27.65, 27.23, 26.70, 26.38, 25.75, 
                 25.44, 25.02, 24.61, 24.09, 23.67, 23.26, 22.74], 'd')
n = len(x)
PositionError    = 0.2
TemperatureError = 0.1
dx = numpy.array(n*[PositionError],'d')
dy = numpy.array(n*[TemperatureError],'d')

#
# Fit with simple Least Squares Method via scipy.optimize.curve_fit, docs in
# http://docs.scipy.org/doc/scipy/reference/generated/scipy.optimize.curve_fit.html
#

# Setup the linear function
def f(x, m, q):
    return m*x + q

# Fit data - ignore errors in this algorithm
# Notice that if errors are all the same (as in this canse), 
# they have no effect on the fit result
popt, pcov = curve_fit(f, x, y, numpy.array([1.,1.]))
m, q       = popt
dm, dq     = numpy.sqrt(pcov.diagonal())
print('m = %f +- %f' % (m, dm))
print('q = %f +- %f' % (q, dq))

#
# Fit with Orthogonal distance regression to use x and y errors
# http://docs.scipy.org/doc/scipy/reference/odr.html
# Is it really useful to use a more complex algorithm?
#

# Setup the linear function
def ff(B, x):
    return B[0]*x + B[1]

linear = odrpack.Model(ff)
mydata = odrpack.RealData(x,y,sx=dx,sy=dy)
myodr  = odrpack.ODR(mydata, linear, beta0=[1., 1.])
myoutput = myodr.run()
#myoutput.pprint() # uncomment for full output

a = myoutput.beta[0]
b = myoutput.beta[1]
da = numpy.sqrt(myoutput.cov_beta[0][0])
db = numpy.sqrt(myoutput.cov_beta[1][1])
print('-------------------------\nODR output:')
print('m = %f +- %f' % (a, da))
print('q = %f +- %f' % (b, db))
print('-------------------------')

#
# Plot data and fit
#

pylab.figure(1)
pylab.subplot(211)
# http://matplotlib.org/1.2.1/api/pyplot_api.html#matplotlib.pyplot.errorbar
pylab.errorbar(x, y, dy, dx, '+' )
pylab.plot(x,f(x, m, q), '-') 
pylab.title('Temperature profile')
pylab.ylabel(u'Temperature [\u00B0C]') # use unicode for 'degree' symbol
pylab.xlabel('Position [cm]')

pylab.subplot(212)
pylab.errorbar(x, y-f(x, m, q), dy, dx, '+' ) # residuals
pylab.plot(x,numpy.array(n*[0.]), '-') # horizontal line at 0
pylab.title('')
pylab.ylabel(u'T - f(x) [\u00B0C]')
pylab.xlabel('Position [cm]')
